begin
  require 'spreadsheet'
rescue LoadError
  ActionController::Base::logger.info 'Duplicate views controller: spreadsheet gem not available'
end

class ListDuplicateViewsController < ApplicationController
  unloadable
  layout 'base'
  before_filter :require_admin

  def index
	  redmine_app_folder=Dir.pwd
	  @list_plugin_folders=true if params[:show_plugins_views] && params[:show_plugins_views] == '1'
	  # array of [view name, view files array in this view [filename,array of plugins names]]
	  @duplicate_views=get_duplicate_views_list(redmine_app_folder)
	  if request.post?
		  if params[:save_legend] && defined?(Spreadsheet::Workbook.new)
			  send_data(save_legend_to_xls, :type => :xls, :filename => 'redmine_duplicate_views.xls')
		  end
	  end

  end

private

  REDMINE_MAIN_APP_NAME = '_redmine_main_app'

	def get_duplicate_views_list(root_path)
		views_files={}

		views_folder="#{root_path}/app/views/"
		views_list=Dir.glob("#{views_folder}*").collect do |d|
			File.basename(d)
		end
		views_list.each do |v|
			views_files[v]={}
			process_view_folder(views_folder,views_folder,v,views_files[v])
		end

		plugins_folder="#{root_path}/vendor/plugins/"
		plugins_list=Dir.glob("#{plugins_folder}*").collect do |d|
			File.basename(d)
		end

		plugins_list.each do |p|
			process_plugin_folder(views_files,p,plugins_folder)
		end

		dup_views=views_files.select do |view,dups|
			r=false
			dups.each_pair do |vf,pa|
				r=true if pa.size > 1
			end
			r
		end

		dup_views.reject! do |e|
			e[1]=e[1].sort do |a,b|
				a[0] <=> b[0]
			end
			e[1].reject! do |a|
				r=false
				if a[1].size > 1
					a[1].reject! do |ra|
						true if ra == REDMINE_MAIN_APP_NAME
					end
				else
					r=true
				end
				r
			end
			r=false
			r=true unless e[1].size > 0
			r
		end

		dup_views.sort! do |a,b|
			a[0] <=> b[0]
		end

		return dup_views
	end

	def process_view_folder(root_dir,base_dir,view_dir,view_files)
		Dir.glob("#{base_dir}#{view_dir}/*").each do |d|
			if File.directory?(d)
				process_view_folder(root_dir,"#{base_dir}#{view_dir}/",File.basename(d),view_files)
			else
				view_files[d.gsub(root_dir,'')]=[REDMINE_MAIN_APP_NAME]
			end
		end
	end

	def process_plugin_folder(views_files,plugin_name,plugins_folder)
		pv_folder="#{plugins_folder}#{plugin_name}/app/views/"
		pv_files={}
		pv_list=Dir.glob("#{pv_folder}*").collect do |d|
			File.basename(d)
		end

		pv_list.each do |v|
			pv_files[v]={}
			process_view_folder(pv_folder,pv_folder,v,pv_files[v])
		end

		pv_files.each_pair do |view_name,data|
			if views_files[view_name]
				data.each_pair do |view_file,data2|
					if views_files[view_name][view_file] || @list_plugin_folders
						views_files[view_name][view_file] = [] unless views_files[view_name][view_file]
						views_files[view_name][view_file] << plugin_name
					end
				end
			else
				if @list_plugin_folders
					views_files[view_name]={}
					data.each_pair do |view_file,data2|
						views_files[view_name][view_file] = [] unless views_files[view_name][view_file]
						views_files[view_name][view_file] << plugin_name
					end
				end
			end
		end
	end

	def save_legend_to_xls
		book = Spreadsheet::Workbook.new

		xcf={}
		xcf[:header_1]=Spreadsheet::Format.new :weight => :bold, :size => 24
		xcf[:header_2]=Spreadsheet::Format.new :size => 20
		xcf[:table_elem]=Spreadsheet::Format.new
		xcf[:table_elem].border=true
		xcf[:table_elem_bold]=Spreadsheet::Format.new :weight => :bold
		xcf[:table_elem_bold].border=true
		xcf[:table_elem_red]=Spreadsheet::Format.new :color => :red
		xcf[:table_elem_red].border=true

		sheet1 = book.create_worksheet(:name => 'Views list (table)')

		sheet1.row(0).replace [l(:table_ldv_header_view_name),l(:table_ldv_header_dup_filename),l(:table_ldv_header_plugin_name)]
		3.times do |i|
			sheet1.row(0).set_format(i,xcf[:table_elem_bold])
		end
		current_row=1

		@duplicate_views.each do |dv|
		  dv[1].each do |v|
				v[1].each do |n|
					sheet1.row(current_row)[0]=dv[0]
					sheet1.row(current_row)[1]=v[0]
					sheet1.row(current_row)[2]=n
					3.times do |i|
						sheet1.row(current_row).set_format(i,xcf[v[1].size > 1 ? :table_elem_red : :table_elem])
					end
					current_row+=1
		    end
		  end
		end

		#sheet1.row(current_row)[0]="Text1\nText2\nText3"

		[30,50,50].each_with_index do |w,idx|
			sheet1.column(idx).width=w
			#sheet1.column(idx).default_format=xcf[:word_wrap_format]
		end

		sheet1 = book.create_worksheet(:name => 'Views list')

		current_row=0

		@duplicate_views.each do |dv|
			sheet1.row(current_row)[0]=dv[0]
			sheet1.row(current_row).set_format(0,xcf[:header_1])
			current_row+=1
		  dv[1].each do |v|
			  sheet1.row(current_row)[0]=v[0]
			  sheet1.row(current_row).set_format(0,xcf[:header_2])
			  current_row+=1
				v[1].each do |n|
					sheet1.row(current_row)[0]=n
					current_row+=1
		    end
		  end
			current_row+=1
		end


		xls_stream = StringIO.new('')
		book.write(xls_stream)

		return xls_stream.string
	end
end

